package owg.lgm.gl;

import com.jogamp.opengl.GL2ES1;
import com.jogamp.opengl.GL3;

public interface DrawCommand
{
	public static class NoOp implements DrawCommand 
	{
		@Override
		public void invoke(GL2ES1 gl) {}
	}
	public static DrawCommand NO_OP = new NoOp();

	public static class DrawArrays implements DrawCommand 
	{
		public PrimitiveType mode;
		public int first, count;
		public DrawArrays(PrimitiveType mode, int first, int count)
		{
			this.mode = mode;
			this.first = first;
			this.count = count;
		}
		protected DrawArrays() {}
		@Override
		public void invoke(GL2ES1 gl) 
		{
			gl.glDrawArrays(mode.value, first, count);
		}
	}
	public static DrawArrays drawArrays(PrimitiveType mode, int first, int count)
	{
		return new DrawArrays(mode, first, count);
	}

	public static class DrawElements implements DrawCommand 
	{
		public BufferObject ibo;
		public PrimitiveType mode;
		public int count;
		public DataType type;
		public long byteOffset;
		public DrawElements(BufferObject ibo, PrimitiveType mode, int count, DataType type, long byteOffset)
		{
			this.ibo = ibo;
			this.mode = mode;
			this.count = count;
			this.type = type;
			this.byteOffset = byteOffset;
		}
		protected DrawElements() {}
		@Override
		public void invoke(GL2ES1 gl) 
		{
			gl.glBindBuffer(GL3.GL_ELEMENT_ARRAY_BUFFER, ibo.getHandle());
			gl.glDrawElements(mode.value, count, type.value, byteOffset);
		}
	}
	public static DrawElements drawElements(BufferObject ibo, PrimitiveType mode, int count, DataType type, long byteOffset)
	{
		return new DrawElements(ibo, mode, count, type, byteOffset);
	}
	
	public abstract void invoke(GL2ES1 gl);
}
