package owg.lgm.gl;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.GMLib;

public class GradientText extends TextImpl
{
	protected ByteBuffer colors;
	protected IntBuffer colorsAsInt;
	protected VertexBufferObjectB cbo;
	
	public int color0, color1;
	
	public GradientText(GL2ES1 gl, BMFontRenderer font, int initialCapacity, int color0, int color1, BufferUsage usage)
	{
		super(gl, font, initialCapacity, usage);
		this.color0 = color0;
		this.color1 = color1;
	}
	
	@Override
	protected void initMeshes()
	{
		cbo = new VertexBufferObjectB(vbo.getGL(), colors, vbo.usage);
		autoDestroy(cbo);
		
		AttribArray positionAttrib = new AttribArray(vbo, VertexAttribute.POSITION, 2, DataType.FLOAT, 4*4, 0);
		AttribArray texCoordAttrib = new AttribArray(vbo, VertexAttribute.TEXTURE_COORDINATE_0, 2, DataType.FLOAT, 4*4, 2*4);
		AttribArray colorAttrib = new AttribArray(cbo, VertexAttribute.COLOR, 4, DataType.UNSIGNED_BYTE, 4, 0);
		for(int i = 0; i<font.pages.length; i++)
		{
			meshes[i] = new Mesh(new DrawCommand.DrawElements(ibo, PrimitiveType.TRIANGLES, 0, DataType.UNSIGNED_SHORT, i*capacity*6*2),
					positionAttrib, texCoordAttrib, colorAttrib);
			autoDestroy(meshes[i]);
		}
	}

	@Override
	protected void allocate(int capacity)
	{
		super.allocate(capacity);
		colors = ByteBuffer.allocateDirect(capacity*4*4).order(ByteOrder.nativeOrder());
		colorsAsInt = colors.asIntBuffer();
	}

	@Override
	public void prepare(int length)
	{
		super.prepare(length);
		colorsAsInt.clear();
	}
	
	@Override
	public void putChar(CharSequence str, int index, int pageIndex, float x0, float y0, float tx0, float ty0, float x1, float y1, float tx1, float ty1)
	{
		int k = verts.position()/4;
		
		verts.put(x0).put(y0).put(tx0).put(ty0);
		verts.put(x1).put(y0).put(tx1).put(ty0);
		verts.put(x0).put(y1).put(tx0).put(ty1);
		verts.put(x1).put(y1).put(tx1).put(ty1);
		
		int c0 = swizzle(GMLib.merge_color(color0, color1, (float)index/str.length()));
		int c1 = swizzle(GMLib.merge_color(color0, color1, (float)(index+1)/str.length()));
		colorsAsInt.put(c0);
		colorsAsInt.put(c1);
		colorsAsInt.put(c0);
		colorsAsInt.put(c1);
		
		ShortBuffer ibuf = pageInds[pageIndex];
		ibuf.put((short)(k+1)); ibuf.put((short)(k  )); ibuf.put((short)(k+2));
		ibuf.put((short)(k+1)); ibuf.put((short)(k+2)); ibuf.put((short)(k+3));
	}

	protected int swizzle(int color)
	{
		return 	color;
	}

	@Override
	public void finish()
	{
		int cCount = colorsAsInt.position()*4;
		colorsAsInt.clear();
		if(buffersChanged)
			cbo.setData(colors);
		else if(cCount > 0)
			cbo.update(0, cCount);
		
		super.finish();
	}
}
