package owg.lgm.in;

import java.util.Arrays;

import com.jogamp.newt.event.KeyEvent;
import com.jogamp.newt.event.MouseEvent;

public class InputState
{	 
	protected static final int maxKey = 256;
	protected static final byte DOWN_MASK = 1;
	protected static final byte PRESSING_MASK = 2;
	protected static final byte RELEASING_MASK = 4;
	
	public final EventQueue<KeyEventType, VK> keyEvents = new EventQueue<>();
	public final EventQueue<MouseEventType, MB> mouseEvents = new EventQueue<>();
	protected final StringBuilder typedChars = new StringBuilder();

	protected final byte[] keys = new byte[maxKey];
	protected final VK[] newtToVK = new VK[maxKey];
	{
		VK[] vals = VK.values();
		for(int i = 0; i<vals.length; i++)
			newtToVK[vals[i].newtKey] = vals[i];
	}
	
	protected final byte[] mouse = new byte[MouseEvent.BUTTON_COUNT+1];
	protected final MB[] newtToMB = new MB[MouseEvent.BUTTON_COUNT+1];
	{
		newtToMB[MouseEvent.BUTTON1] = MB.LEFT;
		newtToMB[MouseEvent.BUTTON2] = MB.MIDDLE;
		newtToMB[MouseEvent.BUTTON3] = MB.RIGHT;
	}
	
	protected int surfaceWidth, surfaceHeight;
	protected boolean focused = false;
	protected boolean mouseInside = false;
	protected boolean mouseEntered = false;
	protected boolean mouseExited = false;
	
	float scrollWheelAmountX, scrollWheelAmountY;

	/**The position of the mouse in the opengl canvas(in window coordinates)*/
	protected int mouseX,mouseY;
	protected boolean mouseMoved;
	
	public void pressKey(int keyCode, char keyChar, int modifiers, boolean isAutoRepeat)
	{
		if(keyChar != '\0')
			typedChars.append(keyChar);
		if(keyCode < 0 || keyCode >= maxKey)
			return;
		if(!isAutoRepeat && (keys[keyCode]&DOWN_MASK) == 0)
		{
			if(newtToVK[keyCode] != null)
				keyEvents.offer(KeyEventType.PRESS, newtToVK[keyCode]);
			keys[keyCode] |= PRESSING_MASK;
			keys[keyCode] |= DOWN_MASK;
		}
	}
	public void releaseKey(int keyCode)
	{
		if(keyCode < 0 || keyCode >= maxKey)
			return;
		if(newtToVK[keyCode] != null)
			keyEvents.offer(KeyEventType.RELEASE, newtToVK[keyCode]);
		{
			keys[keyCode] |= RELEASING_MASK;
			keys[keyCode] &= ~DOWN_MASK;
		}
	}
	public int getModifiers()
	{
		int modifiers = 0;
		if (isKeyDown(KeyEvent.VK_CONTROL))
			modifiers |= KeyEvent.CTRL_MASK;
		if (isKeyDown(KeyEvent.VK_ALT))
			modifiers |= KeyEvent.VK_ALT;
		if (isKeyDown(KeyEvent.VK_SHIFT))
			modifiers |= KeyEvent.VK_SHIFT;
		return modifiers;
	}
	
	public void transfer(InputState next)
	{
		for(int i = 0; i<maxKey; i++)
		{
			next.keys[i] = (byte) (keys[i]&DOWN_MASK);
			if(next.keys[i] != 0 && newtToVK[i] != null)
				keyEvents.offer(KeyEventType.DOWN, newtToVK[i]);
		}
		for(int i = 1; i <= MouseEvent.BUTTON_COUNT; i++)
		{
			next.mouse[i] = (byte) (mouse[i]&DOWN_MASK);
			if(next.mouse[i] != 0 && newtToMB[i] != null)
				mouseEvents.offer(MouseEventType.DOWN, newtToMB[i]);
		}
		next.typedChars.setLength(0);
		
		next.surfaceWidth = surfaceWidth;
		next.surfaceHeight = surfaceHeight;
		next.focused = focused;
		next.mouseInside = mouseInside;
		next.mouseEntered = false;
		next.mouseExited = false;
		next.mouseX = mouseX;
		next.mouseY = mouseY;
		next.mouseMoved = false;
		next.scrollWheelAmountX = 0;
		next.scrollWheelAmountY = 0;

		next.keyEvents.types.clear();
		next.keyEvents.ids.clear();
		next.mouseEvents.types.clear();
		next.mouseEvents.ids.clear();
	}
	
	public boolean isMouseInside()
	{
		return mouseInside;
	}
	public boolean getMouseEntered() {
		return mouseEntered;
	}
	public boolean getMouseExited() {
		return mouseExited;
	}
	public boolean isKeyDown(int keyCode)
	{
		if(keyCode < maxKey)
			return (keys[keyCode]&DOWN_MASK) != 0;
		return false;
	}
	public boolean isKeyPressing(int keyCode)
	{
		if(keyCode < maxKey)
			return (keys[keyCode]&PRESSING_MASK) != 0;
		return false;
	}
	public boolean isKeyReleasing(int keyCode)
	{
		if(keyCode < maxKey)
			return (keys[keyCode]&RELEASING_MASK) != 0;
		return false;
	}
	public float getScrollWheelAmountX()
	{
		return scrollWheelAmountX;
	}
	public float getScrollWheelAmountY()
	{
		return scrollWheelAmountY;
	}
	public boolean isMouseDown(int button)
	{
		if(button <= MouseEvent.BUTTON_COUNT)
			return (mouse[button]&DOWN_MASK) != 0;
		return false;
	}
	public boolean isMousePressing(int button)
	{
		if(button <= MouseEvent.BUTTON_COUNT)
			return (mouse[button]&PRESSING_MASK) != 0;
		return false;
	}
	public boolean isMouseReleasing(int button)
	{
		if(button <= MouseEvent.BUTTON_COUNT)
			return (mouse[button]&RELEASING_MASK) != 0;
		return false;
	}
	public void moveMouse(int x, int y, int modifiers)
	{
		mouseMoved = true;
		mouseX = x;
		mouseY = y;
	}
	public void dragMouse(int x, int y, int button, int modifiers)
	{
		mouseMoved = true;
		mouseX = x;
		mouseY = y;
	}
	public void mouseWheelMoved(float xAmount, float yAmount, int modifiers)
	{
		scrollWheelAmountX += xAmount;
		scrollWheelAmountY += yAmount;
	}
	public void pressMouseButton(int x, int y, int button, int modifiers)
	{
		if(button > MouseEvent.BUTTON_COUNT)
			return;
		if(newtToMB[button] != null)
			mouseEvents.offer(MouseEventType.PRESS, newtToMB[button]);
		{
			mouse[button] |= PRESSING_MASK;
			mouse[button] |= DOWN_MASK;
		}
	}
	public void releaseMouseButton(int x, int y, int button, int modifiers)
	{
		if(button > MouseEvent.BUTTON_COUNT)
			return;
		if(newtToMB[button] != null)
			mouseEvents.offer(MouseEventType.RELEASE, newtToMB[button]);
		{
			mouse[button] |= RELEASING_MASK;
			mouse[button] &= ~DOWN_MASK;
		}
	}
	public void mouseEntered(int x, int y, int modifiers)
	{
		mouseInside = true;
		mouseEntered = true;
	}
	public void mouseExited(int x, int y, int modifiers)
	{
		mouseInside = false;
		mouseExited = true;
	}
	
	public boolean getMouseMoved() {
		return mouseMoved;
	}
	public int getMouseX() {
		return mouseX;
	}
	public int getMouseY() {
		return mouseY;
	}
	public void setFocused(boolean b)
	{
		this.focused = b; 
	}
	public boolean isFocused()
	{
		return focused;
	}
	
	public void setSurfaceSize(int w, int h)
	{
		this.surfaceWidth = w;
		this.surfaceHeight = h; 
	}
	public int getSurfaceWidth()
	{
		return surfaceWidth;
	}
	public int getSurfaceHeight()
	{
		return surfaceHeight;
	}
	
	public EventQueue<KeyEventType, VK> getKeyEvents()
	{
		return keyEvents;
	}
	
	public EventQueue<MouseEventType, MB> getMouseEvents()
	{
		return mouseEvents;
	}
	
	public CharSequence getTypedChars()
	{
		return typedChars;
	}
	
	public void clear()
	{
		keyEvents.types.clear();
		keyEvents.ids.clear();
		mouseEvents.types.clear();
		mouseEvents.ids.clear();
		Arrays.fill(keys, (byte)0);
		Arrays.fill(mouse, (byte)0);
	}
}
