package owg.lgm.plaf;

import owg.lgm.util.FILE;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import jogamp.opengl.util.pngj.ImageInfo;
import jogamp.opengl.util.pngj.PngReader;
import jogamp.opengl.util.pngj.PngWriter;
import owg.lgm.IImage;
import owg.lgm.IImage.Type;

public class PNGIO
{
	public static IImage load(FILE input) throws IOException
	{
		try (InputStream is = input.read()) {
			return load(is, input.getName());
		}
	}
	
	public static IImage load(InputStream is, String fname) throws IOException
	{
		PngReader reader = new PngReader(is, fname);
		reader.setUnpackedMode(true);
		if(reader.imgInfo.indexed)
			throw new IOException(fname+": Indexed color not supported");
		else if(reader.imgInfo.bitDepth != 8)
			throw new IOException(fname+": "+reader.imgInfo.bitDepth+" bit image not supported: Must be 8");
		else if(reader.imgInfo.channels == 4)
				return readPNG(fname, reader, Type.RGBA);
		else if(reader.imgInfo.channels == 3)
			return readPNG(fname, reader, Type.RGB);
		else if(reader.imgInfo.channels == 2)
			return readPNG(fname, reader, Type.LA);
		else if(reader.imgInfo.channels == 1)
			return readPNG(fname, reader, Type.GRAY);
		else
			throw new IOException(fname+": "+reader.imgInfo.channels+" channel image not supported: Must be 1-4.");
	}
	
	protected static IImage readPNG(String fname, PngReader reader, Type type)
	{
		IImage img = new IImage(type, reader.imgInfo.cols, reader.imgInfo.rows, fname);
		byte[] buffer = new byte[reader.imgInfo.samplesPerRow];
		for(int i = 0; i<reader.imgInfo.rows; i++) {
			reader.readRowByte(buffer, i);
			System.arraycopy(buffer, 0, img.data, i*buffer.length, buffer.length);
		}
		reader.end();
		return img;
	}

	public static void save(FILE output, IImage image) throws IOException
	{
		try(OutputStream os = output.write()) {
			save(os, output.getName(), image);
		}
	}
	
	public static void save(OutputStream fo, String fname, IImage image)
	{
		ImageInfo imgInfo = new ImageInfo(image.width, image.height, 8, image.type.bytesPerComponent==4);
		PngWriter pw = new PngWriter(fo, imgInfo, fname);
		try {
			byte[] row = new byte[image.width*image.type.bytesPerComponent];
			for(int y = 0; y<image.height; y++)
			{
				System.arraycopy(image.data, y*row.length, row, 0, row.length);
				pw.writeRowByte(row, y);
			}
		} finally {
			pw.end();	
		}
	}
}
