package owg.competent.soundfile;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class WAV {
	public static int PCM_INT = 1;
	public static int PCM_FLOAT = 3;
	public int RIFF;
	public int fileSize;
	public int WAVE;
	public transient Fmt fmt;
	public transient int dataSize;
	public static class Fmt {	
		public short encoding;
		public short channels;
		public int frameRate;
		public int byteRate;
		public short frameSize;
		public short bitDepth;
	}
	public static WAV prepare(InputStream fin) throws IOException {
		ByteBuffer buf = ByteBuffer.allocate(64).order(ByteOrder.LITTLE_ENDIAN);
		fill(buf, fin, 12);
		WAV wav = new WAV();
		wav.RIFF = buf.getInt();
		wav.fileSize = buf.getInt();
		wav.WAVE = buf.getInt();
		buf.rewind();
		ass("RIFF", wav.RIFF, 0x46464952);
		ass("WAVE", wav.WAVE, 0x45564157);
		while(true) {
			fill(buf, fin, 8);
			int id = buf.getInt(0);
			int sz = buf.getInt(4);
			if(id == 0x20746d66) { //fmt subchunk
				if(sz != 16 && sz != 18)
					throw new IOException("Expected fmt subchunk size to be either 16 or 18, got: "+sz);
				fill(buf, fin, sz);
				wav.fmt = new Fmt();
				wav.fmt.encoding = buf.getShort();
				wav.fmt.channels = buf.getShort();
				wav.fmt.frameRate = buf.getInt();
				wav.fmt.byteRate = buf.getInt();
				wav.fmt.frameSize = buf.getShort();
				wav.fmt.bitDepth = buf.getShort();
				if(sz == 18)
				{
					int cbSize = buf.getShort();
					//Skip extra data, can't use it
					skip(buf, fin, cbSize, id);
				}
				buf.rewind();
			} else if(id == 0x61746164) { //data subchunk
				if(wav.fmt == null)
					throw new IOException("data subchunk appeared before fmt subchunk");
				wav.dataSize = sz;
				return wav;
			} else { //garbage subchunk, skip
				skip(buf, fin, sz, id);
			}
		}
	}
	
	protected static void skip(ByteBuffer buf, InputStream fin, int sz, int chunkID) throws IOException {
		while(sz > 0) {
			int r = fin.read(buf.array(), 0, Math.min(sz, buf.capacity()));
			if(r <= 0)
				throw new EOFException("Unexpected end of WAV file inside subchunk "+chunkID+", size="+sz);
			sz -= r;
		}
	}
	
	protected static void fill(ByteBuffer buf, InputStream fin, int sz) throws IOException {
		buf.limit(sz);
		while(buf.hasRemaining()) {
			int r = fin.read(buf.array(), buf.position(), buf.remaining());
			if(r <= 0)
				throw new EOFException("Unexpected end of WAV file before data subchunk");
			buf.position(buf.position()+r);
		}
		buf.flip();
	}
	protected static void ass(String name, int actual, int expect) throws IOException {
		if(actual != expect)
			throw new IOException("Expected "+name+" to be "+expect+", got: "+actual);
	}
}
