package owg.competent.util;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

public class CAUtil {
	/**Exceptionless close that accepts nulls. Returns false iff the stream was non-null and could not be closed.*/
	public static boolean close(Closeable strm)
	{
		if(strm == null)
			return true;
		try {
			strm.close();
			return true;
		} catch (IOException e) {
			return false;
		}
	}
	/**Exceptionless sleep. Returns true on success (always, unless you use thread interrupts).*/
	public static boolean sleep(long l)
	{
		try {
			Thread.sleep(l);
			return true;
		} catch (InterruptedException e) {
			return false;
		}
	}
	/**Returns the URL for the given resource, as if calling {@link Class#getResource(String)},
	 * but throws {@link RuntimeException} on failure rather than returning null.*/
	public static URL getResource(Class<?> relativeTo, String name) throws RuntimeException
	{
		URL res = relativeTo.getResource(name);
		if(res == null)
			throw new RuntimeException("No resource \""+name+"\" relative to "+relativeTo);
		return res;
	}
	/**Get the URL for a file without implausible exceptions.*/
	public static URL toURL(File programFile)
	{
		try {
			return programFile.toURI().toURL();
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}
}
